#!/usr/local/bin/perl -w
#------------------------------------------------------------------------------
# $Id: createdb.pl,v 1.4 2003/04/29 17:16:45 manager Exp $
#------------------------------------------------------------------------------
# $Log: createdb.pl,v $
# Revision 1.4  2003/04/29 17:16:45  manager
# bug fix: option variables have to be declared ``our'' instead of ``my''
#
# Revision 1.3  2003/01/24 00:02:56  manager
# adapted to MySQL 4.0.9:
# - additional table files
# - more columns in the tables of the meta database
#
# Revision 1.2  2002/10/08 16:12:11  manager
# - use strict
# - manpage added
#
# Revision 1.1  2002/10/08 15:01:00  manager
# Initial revision
#
#------------------------------------------------------------------------------
# create a new database of the mysql database system
# afb 5/97
#------------------------------------------------------------------------------

=head1 NAME

createdb -- create a new MySQL database

=head1 SYNOPSIS

B<createdb> [B<-d> I<dbname>] [B<-p> I<port>] [B<-s> I<socket>] I<datadir>

=head1 DESCRIPTION

B<createdb> configures and creates a new MySQL database below
F<datadir>. Following options are supported:

=over 4

=item B<-d> I<dbname>

By default, two databases are created, B<mysql> with meta-informations,
and B<main> for the application database. If I<dbname> is provided,
it replaces B<main>. Note that B<mysql> must not be given to avoid
a conflict with the meta database.

=item B<-p> I<port>

This option allows to specify the port the database server should
listen to.  By default, a port is selected by random and stored within
I<datadir>/F<setenv>.

=item B<-s> I<socket>

This option allows to specify a named socket the database server
should listen to. By default this is I<datadir>/F<socket>.

=back

=head1 AUTHOR

Andreas Borchert

=cut

use strict;
use English;
use Fcntl;
use File::Copy;
use Getopt::Mixed;
use IO::File;
use Mysql;

#------------------------------------------------------------------------------
# tunables
#------------------------------------------------------------------------------
my $home = "/usr/local/mysql"; # home directory of mysql distribution
my $bindir = "$home/bin";      # mysqladmin is located here
my $mysqld = "$bindir/mysqld";
my $mysqladmin = "$bindir/mysqladmin";
my $defdir = "$home/data/mysql";


#------------------------------------------------------------------------------
# command line
#------------------------------------------------------------------------------
my $cmdname = $0; $cmdname =~ s{.*/}{};
our $opt_d = undef; our $opt_p = undef; our $opt_s = undef;
my $usage = "Usage: $cmdname [-d dbname] [-p port] [-s socket] datadir\n";
Getopt::Mixed::init("d=s p=i s=s port>p socket>s dbname>d");
$Getopt::Mixed::badOption = sub {
   my ($index, $option, $problem) = @_;
   $problem = 'unrecognized' unless $problem;
   die "$cmdname: $problem option `$option'\n$usage";
};
Getopt::Mixed::getOptions();
die $usage unless @ARGV == 1;
my $datadir = $ARGV[0];
my $dbname;
if (defined($opt_d)) {
   $dbname = $opt_d;
} else {
   $dbname = "main";
}
my $port;
if (defined($opt_p)) {
   $port = $opt_p;
} else {
   $port = int(rand 10000) + 10000; # take a random port by default
}
my $socket;
if (defined($opt_s)) {
   $socket = $opt_s;
} else {
   $socket = "$datadir/socket";
}
my $login = getpwuid($REAL_USER_ID);

#------------------------------------------------------------------------------
# setup new database directories
#------------------------------------------------------------------------------
die "$cmdname: $datadir exists already\n" if -d $datadir;
die "$cmdname: unable to create $datadir\n" unless mkdir($datadir, 0777);
create_dbdir("mysql");
#create_dbdir($dbname);

#------------------------------------------------------------------------------
# start mysql daemon
#------------------------------------------------------------------------------
my @args = ($mysqld,
   "--basedir=$home",
   "--datadir=$datadir",
   "--port=$port",
   "--socket=$socket",
   "--skip-grant-tables",
);
my $server_pid = spawn(@args);
die "$cmdname: unable to invoke " . join(" ", @args) . "\n"
   unless defined $server_pid;
sleep(3);

#------------------------------------------------------------------------------
# setup meta database of mysql
#------------------------------------------------------------------------------
my @tables = qw(db host user func);
foreach my $table (@tables) {
   my $src = "$defdir/$table.frm";
   my $dest = "$datadir/mysql/$table.frm";
   copy($src, $dest) or
      die "$cmdname: unable to copy from $src to $dest\n";

   for my $suffix (qw(MYD MYI)) {
      my $src = "$defdir/$table.$suffix";
      if (-f $src) {
	 my $dest = "$datadir/mysql/$table.$suffix";
	 copy($src, $dest) or
	    die "$cmdname: unable to copy from $src to $dest\n";
      }
   }
}

#------------------------------------------------------------------------------
# it's ugly but some parameters must be set by environment variables
#------------------------------------------------------------------------------
$ENV{MYSQL_UNIX_PORT} = $socket;
$ENV{MYSQL_TCP_PORT} = $port;

#------------------------------------------------------------------------------
# setup grant tables
#------------------------------------------------------------------------------
my @stmts = ();
foreach my $table (@tables) {
   push(@stmts, "delete from $table");
}

my $sql_statement = "insert into db values ('localhost', '$dbname', '$login'";
foreach (1..12) { $sql_statement .= ", 'Y'" }
$sql_statement .= ")";
push(@stmts, $sql_statement);


$sql_statement = "insert into host values ('localhost', '$dbname'";
foreach (1..12) { $sql_statement .= ", 'Y'" }
$sql_statement .= ")";
push(@stmts, $sql_statement);

$sql_statement = "insert into user values ('localhost', '$login', ''";
foreach (1..21) { $sql_statement .= ", 'Y'" }
$sql_statement .= ", '','','','',0,0,0";
$sql_statement .= ")";
push(@stmts, $sql_statement);

my $db = Mysql->connect();
$Mysql::db_errstr = "What happened?";
$db->selectdb("mysql");
foreach my $stmt (@stmts) {
   die "$cmdname: $stmt: $Mysql::db_errstr\n"
      unless $db->query($stmt);
}

#------------------------------------------------------------------------------
# create some files
#------------------------------------------------------------------------------
my $env = new IO::File "$datadir/setenv", O_WRONLY|O_CREAT|O_TRUNC, 0644;
if (defined $env) {
   print $env "MYSQL_UNIX_PORT=$socket\n";
   print $env "MYSQL_TCP_PORT=$port\n";
   $env->close;
}

#------------------------------------------------------------------------------
# create new database
#------------------------------------------------------------------------------
die "$cmdname: unable to create database $dbname\n"
   unless system($mysqladmin, "create", $dbname) == 0;
die "$cmdname: unable to shutdown daemon\n"
   unless system($mysqladmin, "shutdown") == 0;
sleep(3); # give shutdown some time

#------------------------------------------------------------------------------
sub spawn {
   my @args = @_;
   my $pid;
   unless ($pid = fork) {
      exec(@args);
      die "$cmdname: unable to exec " . join(" ", @args) . "\n";
   }
   return $pid;
}

sub create_dbdir {
   my $dbname = shift;
   die "$cmdname: unable to create $datadir/$dbname\n"
      unless mkdir("$datadir/$dbname", 0777);
}
